import junit.framework.Assert;
import org.junit.Test;

import javax.swing.text.html.HTMLDocument;

import java.util.stream.Stream;

import static org.junit.Assert.*;

public class CheckbookTest {
    Checkbook checkbook;
    Transaction debit;
    Transaction credit;
    Payee payeeDebit;
    Payee payeeCredit;
    Payee owner;

    @org.junit.Before
    public void setUp() throws Exception {
        this.owner = new Payee();
             owner.setGivenName("Nick Satinover");
        this.payeeCredit = new Payee();
             payeeCredit.setGivenName("Credit Payee");
        this.payeeDebit = new Payee();
             payeeDebit.setGivenName("Debit Payee");
        this.debit = new Transaction("This is a Debit", payeeDebit, TransactionType.DEBIT, 100.50);
        this.credit = new Transaction("This is a Credit", payeeCredit, TransactionType.CREDIT, 100.50);
        this.checkbook = new Checkbook(owner, 1000.00);
    }

    @Test
    public void test1Size(){
        checkbook.add(debit);
        int expected = 1;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Size(){
        checkbook.add(debit);
        checkbook.add(credit);
        int expected = 2;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Size(){
        checkbook.add(debit);
        checkbook.remove(debit);
        int expected = 0;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }
//
    @Test
    public void test1IsEmpty(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.isEmpty();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2IsEmpty(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.isEmpty();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3IsEmpty(){
        boolean expected = true;
        boolean actual = checkbook.isEmpty();
        Assert.assertEquals(expected, actual);
    }
//

    @Test
    public void test1Contains(){
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.contains(debit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Contains(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.contains(credit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Contains(){
        checkbook.add(credit);
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.contains(credit);
        boolean actual2 = checkbook.contains(debit);
        Assert.assertEquals(expected, actual);
        Assert.assertEquals(expected, actual2);
    }

    @Test
    public void test1Add(){
        checkbook.add(debit);
        int expected = 1;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Add(){
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.contains(debit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Add(){
        checkbook.add(credit);
        boolean expected = true;
        boolean actual = checkbook.contains(credit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1Remove(){
        checkbook.add(debit);
        checkbook.remove(debit);
        int expected = 0;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Remove(){
        checkbook.add(debit);
        boolean expected = false;
        boolean actual = checkbook.remove(credit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Remove(){
        checkbook.add(debit);
        boolean expected = true;
        boolean actual = checkbook.remove(debit);
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1Iterator(){
        checkbook.add(debit);
        checkbook.add(credit);
        String expected = "Itr";
        String actual = checkbook.iterator().getClass().getSimpleName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Iterator(){
        checkbook.add(debit);
        String expected = "This is a Debit";
        String actual = checkbook.iterator().next().getMemo();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Iterator(){
        checkbook.add(debit);
        Payee expected = payeeDebit;
        Payee actual = checkbook.iterator().next().getPayee();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1ToArray(){
        checkbook.add(debit);
        checkbook.add(credit);
        String expected = "Transaction[]";
        String actual = checkbook.toArray().getClass().getSimpleName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2ToArray(){
        checkbook.add(debit);
        double expected = 100.50;
        Transaction[] ret = checkbook.toArray();
        double actual = ret[0].getAmount();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3ToArray(){
        checkbook.add(debit);
        checkbook.add(credit);
        String expected = payeeCredit.getGivenName();
        Transaction[] ret = checkbook.toArray();
        String actual = ret[1].getPayee().getGivenName();
        Assert.assertEquals(expected, actual);
    }






    @org.junit.After
    public void tearDown() throws Exception {
        System.out.println("Running: TearDown");
        checkbook = null;
        assertNull(checkbook);
        if (checkbook == null) System.out.println("Checkbook null, TearDown complete.");
        else System.out.println("TearDown NOT successful, checkbook did not evaluate to NULL");
    }
}